from django import template
from django.template import defaulttags
from django.template.base import Node
from django.templatetags.static import do_static

register = template.Library()

@register.tag
def fullurl(parser, token):
    """Return an absolute URL (including the scheme and domain) matching the
    given view with its parameters.

    This is meant to be identical to the built-in tag `url`, except that it
    always returns an absolute URL with the scheme and authority parts.

    For example, take this `url` tag:

        {% url "articles:article" slug="hello" %}

    This could return:

        /articles/hello

    This is considered an absolute URL because it begins with a forward-slash,
    however, it is not an absolute absolute URL, because it does not include
    the scheme and authority parts.

    Compare with this `fullurl` tag:

        {% fullurl "articles:article" slug="hello" %}

    This returns:

        http://example.com/articles/hello

    """
    return FullURLNode(defaulttags.url(parser, token))


@register.tag
def fullstatic(parser, token):
    return FullStaticNode(do_static(parser, token))


@register.simple_tag(takes_context=True)
def buildfullurl(context, url):
    """Converts relative URL to absolute.

    For example:

        {% buildfullurl article.get_absolute_url %}

    or:

        {% buildfullurl "/custom-url/" %}

    """
    return context.request.build_absolute_uri(url)


class FullURLNode(Node):
    def __init__(self, subject):
        self._subject = subject

    def render(self, context):
        url = self._subject.render(context)
        return context.request.build_absolute_uri(url)

class FullStaticNode(Node):
    def __init__(self, subject):
        self._subject = subject

    def render(self, context):
        url = self._subject.render(context)
        return context.request.build_absolute_uri(url)



from django.conf.urls import url
from django.test import TestCase
from django.test.client import RequestFactory
from django.template import Template, RequestContext

urlpatterns = [
    url(r'^foobar$', lambda r: None, name='example'),
    url(r'^foobar/(.+)$', lambda r: None, name='example2'),
    url(r'^foobar/(?P<slug>.+)$', lambda r: None, name='example3'),
]

class FullURLTestCase(TestCase):
    TEMPLATE = Template('{% load fullurl %}{% fullstatic "hello" }')

    def setUp(self):
        factory = RequestFactory()
        request = factory.get('/')
        self.request_context = RequestContext(request, {})

    def test_fullstatic(self):
        template = Template('{% load fullurl %}{% fullstatic "one/two/hello.jpg" %}')
        rendered = template.render(self.request_context)
        self.assertEquals(rendered, 'http://testserver/static/one/two/hello.jpg')

    def test_fullstatic_with_external_domain(self):
        template = Template('{% load fullurl %}{% fullstatic "one/two/hello.jpg" %}')
        with self.settings(STATIC_URL='http://example.com/static/'):
            rendered = template.render(self.request_context)
            self.assertEquals(rendered, 'http://example.com/static/one/two/hello.jpg')

    def test_fullurl(self):
        template = Template('{% load fullurl %}{% fullurl "example" %}')
        with self.settings(ROOT_URLCONF=__name__):
            rendered = template.render(self.request_context)
            self.assertEquals(rendered, 'http://testserver/foobar')

    def test_fullurl_with_args(self):
        template = Template('{% load fullurl %}{% fullurl "example2" "welcome" %}')
        with self.settings(ROOT_URLCONF=__name__):
            rendered = template.render(self.request_context)
            self.assertEquals(rendered, 'http://testserver/foobar/welcome')

    def test_fullurl_with_kwargs(self):
        template = Template('{% load fullurl %}{% fullurl "example3" slug="welcome" %}')
        with self.settings(ROOT_URLCONF=__name__):
            rendered = template.render(self.request_context)
            self.assertEquals(rendered, 'http://testserver/foobar/welcome')

    def test_buildfullurl_custom(self):
        template = Template('{% load fullurl %}{% buildfullurl "/custom-url/" %}')
        rendered = template.render(self.request_context)
        self.assertEquals(rendered, 'http://testserver/custom-url/')



from setuptools import setup, find_packages
import os

BASE_DIR = os.path.abspath(os.path.dirname(__file__))

readme = open(os.path.join(BASE_DIR, 'README.rst'), 'rb').read().decode('UTF-8')

setup(
    name='django-fullurl',
    version='1.2',
    packages=find_packages(exclude=['testproject']),
    author='David D Lowe',
    author_email='daviddlowe.flimm@gmail.com',
    description='Adds three template tags to Django: `fullurl`, `fullstatic` and `buildfullurl`. The template tag `fullurl` acts just like `url`, but it always prints absolute URLs with scheme and domain',
    long_description=readme,
    long_description_content_type='text/x-rst',
    license='mit',
    keywords=['django'],
    install_requires=[
        'Django>=1.8',
    ],
    url='https://github.com/Flimm/django-fullurl',
    project_urls={
        'GitHub': 'https://github.com/Flimm/django-fullurl',
        'Change log': 'https://github.com/Flimm/django-fullurl/blob/master/CHANGELOG.rst',
        'Say Thanks!': 'https://saythanks.io/to/Flimm',
    },
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, <4',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Framework :: Django :: 3.2',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'License :: OSI Approved :: MIT License',
    ],
)
